"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
function sync(fsAdapter, root, options) {
    const names = fsAdapter.readdir(root);
    const entries = [];
    for (const name of names) {
        const fullPath = path.join(root.toString(), name);
        if (options.preFilter && !options.preFilter(name, fullPath)) {
            continue;
        }
        const stats = fsAdapter.stat(fullPath, {
            followSymlinks: options.followSymlinks,
            throwErrorOnBrokenSymlinks: options.throwErrorOnBrokenSymlinks
        });
        const entry = makeDirEntry(name, fullPath, stats, options);
        if (options.filter && !options.filter(entry)) {
            continue;
        }
        entries.push(entry);
    }
    if (options.sort) {
        return entries.sort(options.sort);
    }
    return entries;
}
exports.sync = sync;
function async(fsAdapter, root, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const names = yield fsAdapter.readdir(root);
        const fullPaths = names.map((name) => path.join(root.toString(), name));
        let filteredNames = [];
        let filteredFullPaths = [];
        if (options.preFilter) {
            for (let index = 0; index < names.length; index++) {
                const name = names[index];
                const fullPath = fullPaths[index];
                if (options.preFilter(name, fullPath)) {
                    filteredNames.push(name);
                    filteredFullPaths.push(fullPath);
                }
            }
        }
        else {
            filteredNames = names;
            filteredFullPaths = fullPaths;
        }
        const promises = filteredFullPaths.map((fullPath) => fsAdapter.stat(fullPath, {
            followSymlinks: options.followSymlinks,
            throwErrorOnBrokenSymlinks: options.throwErrorOnBrokenSymlinks
        }));
        const stats = yield Promise.all(promises);
        const entries = [];
        for (let index = 0; index < filteredNames.length; index++) {
            const name = filteredNames[index];
            const fullPath = filteredFullPaths[index];
            const entry = makeDirEntry(name, fullPath, stats[index], options);
            if (options.filter && !options.filter(entry)) {
                continue;
            }
            entries.push(entry);
        }
        if (options.sort) {
            return entries.sort(options.sort);
        }
        return entries;
    });
}
exports.async = async;
function makeDirEntry(name, full, stats, options) {
    const entry = {
        name,
        path: full,
        ino: stats.ino,
        isDirectory: stats.isDirectory(),
        isFile: stats.isFile(),
        isSymlink: stats.isSymbolicLink()
    };
    if (options.stats) {
        entry.stats = stats;
    }
    return entry;
}
exports.makeDirEntry = makeDirEntry;
